#' Makes run subsets for RPUs with  multiple large outlets for parallelization
#' @param terminal_path : (Character) Terminal Path ID 
#' @param nhd : (SF, data frame) nhd flowlines formatted for hyRefactor
#' @param reconciled : (SF, data frame) reconciled flowlines
#' @param divides : (SF, data frame) output of divides operation
#' @param outlets : (SF, data frame) outlet points for divides/reconciled
#' @events : (SF, data frame) outlet points for events generated and used to split flowlines/catchments
#' @return List of SF layers specific to each terminal path which are are inputs to aggregate_catchments
make_run_subsets <- function(terminal_path, nhd, reconciled, divides, outlets) {
  
  requireNamespace("sf", quietly = TRUE)
  
  # Filter nhd by terminal path
  nhd_sub <- dplyr::filter(nhd, TerminalPa == terminal_path)
  
  comid <- lapply(strsplit(reconciled$member_COMID, split = ","), 
                  function(x) as.integer(x)[1])
  
  reconciled_sub <- dplyr::filter(reconciled, comid %in% nhd_sub$COMID)
  
  divides_sub <- dplyr::filter(divides, ID %in% reconciled_sub$ID)
  
  # if no divides return nothing
  if(nrow(divides_sub) == 0) return(NULL)
  
  divides_sub <- nhdplusTools:::check_valid(divides_sub)
  
  outlets_sub <- dplyr::filter(outlets, ID %in% reconciled_sub$ID)
  
  # return output ready to go into agg_Cats
  list(nhd = nhd_sub, reconciled = reconciled_sub, 
       divides = divides_sub, outlets_fin = outlets_sub)
}

run_agg2 <- function(rs) {
  ##########################################
  # Slightly modified version of Aggregate_catchments that writes features of terminal path run subset where there was a 
  #          processing error, while not interrupting the aggregation of remaining run subsets
  # 
  # Arguments:
  #   rs : (list) list of objects needed for aggregated catchments and linked to a 
  #         single terminal path
  #   rs$Objects: 
  #     terminal_path : (Character) Terminal Path ID 
  #     nhd : (SF, data frame) nhd flowlines formatted for hyRefactor
  #     reconciled : (SF, data frame) reconciled flowlines
  #     divides : (SF, data frame) output of divides operation
  #     gf_outlets : (SF, data frame) outlet points for divides/reconciled
  #
  # Returns:
  #     Aggregated catchments for a given terminal path
  #
  ########################################
  
  print(rs$number)
  message(paste(Sys.time(), "running", rs$number, "with", nrow(rs$reconciled), "rows \n"))
  
  tryCatch({
    # Run aggregate catchments
    hyRefactor::aggregate_catchments(flowpath = rs$reconciled,
                                     divide = rs$divides,
                                     outlets = rs$outlets,
                                     da_thresh = 1,
                                     only_larger = TRUE)
    },
    error = function(e) {
      
      write(toString(e), file.path("temp", paste0(unique(rs$nhd$RPUID),
                                                  "_", rs$number, ".txt")), append=TRUE)
      # write out run subset where there was an issue for inspection
      message("ERROR \n")
      error_gpkg <- file.path("temp", paste0(unique(rs$nhd$RPUID),
                                                       "_", rs$number, ".gpkg"))
      sf::write_sf(rs$nhd, error_gpkg, "nhd")
      sf::write_sf(rs$reconciled, error_gpkg, "reconciled")
      sf::write_sf(rs$divides, error_gpkg, "divides")
      sf::write_sf(rs$outlets, error_gpkg, "outlets")
      return(NA)
    })
}


